#!/bin/bash
# routine to
# animate.bash -a 4 -n 40 -u -p vib_0000.dat -i snap.xyz 
typeset -i i j k m ka ii jj kk ja

ifi=snap.xyz # file containing positions 
pfi=vib_0066.dat # file containing vibrations (eigenmodes)

anzsh=6
ampp=0.5
unq=false
ka=0
ela="1 0 0 \n 0 1 0 \n 0 0 1\n"  ; lela=false
volc=true # generate volume conserving distortions
off=0     # generate positions out of equilibrium pos

while getopts d:i:p:a:n:o:ue:v opt
do
 case $opt in 
  i) ifi="${OPTARG}"
  ;;
  p) pfi="${OPTARG}"
  ;;
  a) ampp="${OPTARG}"   # amplitude of the vibration
  ;;
  n) anzsh="${OPTARG}"  # number of frames to be generated
  ;;
  u) unq=true           # generate only unique positions (for energy landscape)
  ;;
  o) ka="${OPTARG}"+1    # index offset
  ;;
  e) ela="${OPTARG}" ; lela=true    # distortions of unit cell
  ;;
  v) volc=false          # generate volume non-conserving distortions
  ;;
  d) off="${OPTARG}"      # generate volume non-conserving distortions
  ;;
 esac
done ; OPTID=0

if grep "uc" $ifi >> /dev/null 2> /dev/null ; then
 ucs=`grep "uc" $ifi | awk -F ':' '{ print $2}'`
else 
 echo "WARNING: Should take into account the units (fractional coordinates) of the ${pfi} file"
 echo "work with cubic unit cell"
 ucs="1 0 0 \n 0 1 0 \n 0 0 1\n"
fi
j=0 
for k in  1 2 3 ; do
 c=(`echo -e "${ucs}" | awk -v a=${k} '{ print $a  }'`)
 j=${j}+${#c[*]}
done
# Check Unit cell
echo "#ucs: ${j}, ucs: "${ucs}
[ ${j} -eq  9 ] || { echo "ERROR: Unit-cell not correct!" ; exit ; } 

mass=(`sed '1,2d ; s/ /  /g ; s/H /1.0079/;s/He /4.0026/;s/Li /6.941/;s/Be /9.0122/;s/B /10.811/;s/C /12.0107/;s/N /14.0067/;s/O /15.9994/;s/F /18.9984/;s/Ne /20.1797/;s/Na /22.9897/;s/Mg /24.305/;s/Al /26.9815/;s/Si /28.0855/;s/P /30.9738/;s/S /32.065/;s/Cl /35.453/;s/Ar /39.948/;s/K /39.0983/;s/Ca /40.078/;s/Sc /44.9559/;s/Ti /47.867/;s/V /50.9415/;s/Cr /51.9961/;s/Mn /54.938/;s/Fe /55.845/;s/Co /58.9332/;s/Ni /58.6934/;s/Cu /63.546/;s/Zn /65.39/;s/Ga /69.723/;s/Ge /72.64/;s/As /74.9216/;s/Se /78.96/;s/Br /79.904/;s/Kr /83.8/;s/Rb /85.4678/;s/Sr /87.62/;s/Y /88.9059/;s/Zr /91.224/;s/Nb /92.9064/;s/Mo /95.94/;s/Ru /101.07/;s/Rh /102.9055/;s/Pd /106.42/;s/Ag /107.8682/;s/Cd /112.411/;s/In /114.818/;s/Sn /118.71/;s/Sb /121.76/;s/Te /127.6/;s/I /126.9045/;s/Xe /131.293/;s/Cs /132.9055/;s/Ba /137.327/;s/La /138.9055/;s/Ce /140.116/;s/Pr /140.9077/;s/Nd /144.24/;s/Sm /150.36/;s/Eu /151.964/;s/Gd /157.25/;s/Tb /158.9253/;s/Dy /162.5/;s/Ho /164.9303/;s/Er /167.259/;s/Tm /168.9342/;s/Yb /173.04/;s/Lu /174.967/;s/Hf /178.49/;s/Ta /180.9479/;s/W /183.84/;s/Re /186.207/;s/Os /190.23/;s/Ir /192.217/;s/Pt /195.078/;s/Au /196.9665/;s/Hg /200.59/;s/Tl /204.3833/;s/Pb /207.2/;s/Bi /208.9804/;s/Th /232.0381/;s/Pa /231.0359/;s/U /238.028/ ' $ifi | awk '{ print $1 " "$1 " " $1 }'`)

ofi=`echo ${pfi} | sed 's/\.dat// ; s/vib/run/'`

. integer_lib

function otp { # write file to output
local i j inxl vec ofi c ucss

inxl=$1
vec=($2)
ofi=$3
ste=$4 #step number
amp=$5 #amplitude
ucss=$6 # unit cell
maxx=$7 # equivalent mass

i=0
inxl=${inxl} ; c=(${inxl}) ; 


echo "`echo -e "${#c[*]} \n ${amp} E ${ste} uc: "`" ${ucss} "#mass" ${mass} > $ofi
for fon in $inxl ; do
 ze="${fon}"
 for k in 0 1 2 ; do
  ze="${ze} ${vec[${i}]}"
  i=${i}+1
 done
 echo $ze >> $ofi
done
#cat $ofi

}

function add { # add two vectors
local i wec vec  

wec=($1)
vec=$2 
typeset -i i 

co=${ofi}.$$ ; i=0
#echo ${vec}
for fon in ${vec} ; do
 { echo ${i} `echo "scale=8 ; (${fon})+(${wec[${i}]})" | bc` >> .hef.${co} ; } &
 i=${i}+1
done
wait 
sort -n .hef.${co} | awk '{ print $2}' ; rm .hef.${co}
}

function mult { # multiplication scalar with vector
local i vec c
vec=$2  
c=$1
typeset -i i 

co=$$ ; i=0
#echo ${vec}
for fon in ${vec} ; do
 { echo ${i} `echo "scale=8 ; (${fon})*(${c})" | bc` >> .hef.${co} ; } &
 i=${i}+1
done
wait
sort -n .hef.${co} | awk '{ print $2}' ; rm .hef.${co}
}

function vmult { # scalar product: vector multiplication with vector
local i wec vec  hstr

wec=($1)
vec=$2 
typeset -i i 

co=${ofi}.$$ ; i=0
#echo ${vec}
hstr="0"
for fon in ${vec} ; do
 hstr="${hstr}+(${fon})*(${wec[${i}]})"
 i=${i}+1
done
echo "scale=8 ; ${hstr}" | bc
}

function mvmult { # matrix-vector product: multiply vector with matrix
local i mat matl vec hstr

mat=$1
vec=$2 
typeset -i i 

i=1
hstr=`echo -e  ${mat}`
c=(`echo -e "${hstr}" | awk '{ print $1 }'`) 
n=${#c[*]}
hstr=""
while [ ${i} -le ${n} ] ; do
 matl=`echo -e "${mat}" | head -${i} | tail -1`
 hstr="${hstr} "`vmult  "${matl}" "${vec}"`
 i=${i}+1
done
echo "${hstr}" 
}

function mmmult { # matrix-matrix product: multiply matrix with matrix
local i k mat1 matl mat2 matm  hstr m n c mh

mat1=$1
mat2=$2 
typeset -i i  k m n


#m: anzahl zeilen in m1
c=(`echo -e "${mat1}" | awk '{ print $1 }'`) 
m=${#c[*]}
#n: anzahl spalten in m2
mh=(`echo -e "${mat2}" | wc`) 
n=${mh[0]}-1
echo "${hstr} m: ${m} n: ${n} c:${c}" 
i=1 ; hstr=""
while [ ${i} -le ${m} ]  ; do
 matm=`echo -e "${mat1}" | head -${i} | tail -1`
# echo "matm"
  k=1
  while [ ${k} -le ${n} ]  ; do
   matl=`echo -e "${mat2}" | awk -v a=$k '{ print $a }'`   
 #  echo "l: ${matl}" "m: ${matm}"
   hstr="${hstr} "` vmult  " ${matl}" "${matm}"`
   k=${k}+1
  done
   hstr="${hstr}\n"
 i=${i}+1
done
echo "${hstr}" # m: ${m} n: ${n} c:${c}" 
}

pvec=`sed 1,2d ${ifi} | awk '{ print $2 " " $3 " " $4 }'` # read in positions
#pvec=`mult 0.529177249  "${pvec}"`                       # conversion bohr to angstrom
ivec=`sed 1,2d ${ifi} | awk '{ print $1 }'`               # list containing atom names
vvec=`sed -n '/#/d ; /[0-9]/p'  ${pfi}`                   # vector containing vibrations
 
 
#echo ${ivec}
#echo ${vvec}


#add "1 2 3"  "4 5 6"
#add "1.1 2.1 3.1 4.1 5.1 6.1"  "20.5 19.5 18.5 17.5 16.5 15.5"
#echo "fiad"
#mult 4 "1 2 3"   
#vmult "1 2 3"  "4 5 6"
#echo "nw"
#mult 4 "1.1 2.1 3.1 4.1 5.1 6.1"
#add  "${vvec}"  "${pvec}"
#otp "${ivec}" "${pvec}" mynewf.xyz 00 0.001  "${ucs}"
#mvmult "1.1 2.1  3.1 \n 4.1 5.1 6.1\n"  "1 2 3"
#mvmult "1.1 2.1  3.1 \n 4.1 5.1 6.1\n 7.1 8.1 9.1\n"  "-0.1 -0.2 -.3"
#mvmult  "1.000000000006 0 0 \n 0 1.000000000006 0\n 0 0 1.000000000006 \n"   "0 0 -.28752607"
#mmmult "1.1 2.1  3.1 \n 4.1 5.1 6.1\n"  "1.1 2.1  3.1 \n 4.1 5.1 6.1\n 4.1 5.1 6.1\n"
echo -e "ela:  ${ela}"
 

hpvec=(${pvec})
m=${#hpvec[*]}/3 
echo "number atoms: ${m}"

jj=0 ;  
if [ "${vvec}" == "" ] ; then      # no displacement vector provided
 vvec="" ;  j=0 ;  avev=("" 0 0 0) # generate displacement and remove displacement
 while [ ${j} -lt ${m} ]  ; do     # of center of gravity 
  kk=${jj}+1 ; ii=${jj}+2 
  lpvec="${hpvec[${jj}]} ${hpvec[${kk}]} ${hpvec[${ii}]}"
  for k in 1 2 3 ; do 
   zi=(`echo -e "$ela" | head -${k} | tail -1`)  
   kk=${k}-1
   zi[${kk}]="(${zi[${kk}]}-1)"
 #echo "vmult  ${lpvec}   ${zi[*]}"
   avev[0]=`vmult "${lpvec}" "${zi[*]}"`
   vvec="${vvec} ${avev[0]}"
   avev[${k}]="${avev[${k}]}+(${avev[0]})"
  done 
  vvec="${vvec}\n"
  j=${j}+1 ; jj=${jj}+3 ;
 done
# echo -e "vvec: ${vvec}" 
 for k in 1 2 3 ; do 
  avev[${k}]=`echo "scale=8 ; -(${avev[${k}]})/${m}" |bc`
 done 
 avev[0]=""
# echo "avev: ${avev[*]}"
 j=1;  hvvec=""
 while [ ${j} -le ${m} ]  ; do # 
  zi=`echo -e "${vvec}" | head -${j} | tail -1`
  hvvec="${hvvec}"`add "${zi}" "${avev[*]}"`"\n"
 #echo "hvec: " ${hvvec}
  j=${j}+1 
 done
 vvec=`echo -e ${hvvec}`
# echo "vvec: ${vvec}" 
 pfi=`pwd | awk -F / '{ print $NF }'`".dat"
 ofi=`echo ${pfi} | sed 's/\.dat// ; s/vib/run/'`
fi
hpe=(${vvec})
 

mx=0
for j in 1 2 3 ; do   # determine longest unit cell vector
 zi=`echo -e "$ucs" | head -${j} | tail -1`
 zi=`vmult "${zi}"  "${zi}"`
 echo "${zi} > ${mx}" | bc >> /dev/null &&  mx=${zi}
# echo "zi: ${zi} mx: ${mx}"
done
mx=`echo "scale=12 ; 1/sqrt(${mx})" | bc`
echo "mx: ${mx}"

nucs=""
for j in 1 2 3 ; do   # rescale unit cell
 zi=`echo -e "${ucs}" | head -${j} | tail -1`
 zi=`mult ${mx} "${zi}"`
 nucs="${nucs}"`echo ${zi}`"\n"
done
echo "ucs: ${ucs}"
echo "nucs: ${nucs}" 

j=0 ; nhpe=()
while [ ${j} -lt ${m} ]  ; do # multiply vectors with unit cell
 lv=""
 for k in 0 1 2 ; do
  k=${k}+${j}*3
  lv="${lv} ${hpe[${k}]}"
 done 
 nhpe[${j}]=`mvmult "${nucs}"  "${lv}"` # use rescaled unit cell
 j=${j}+1
done
#echo ${hpe[*]}
nhpe=(${nhpe[*]})
#echo ${nhpe[*]}
echo ${#nhpe[*]} 

k=0 ; mum=0
for fon in ${vvec} ; do   
 mum="${mum}+(${fon})*(${fon})*${mass[k]}"
 k=${k}+1 
done
mass=`echo "scale=12 ; (${mum})*1822.88839 " | bc`
echo "mass: ${mass}"
lopfi=`ls ${pfi}`
if [ "${lopfi}" == "" ] && ${lela} ; then
 echo -e "# mass: ${mass} \n ${vvec}" > ${pfi}
fi 

vvec=${nhpe[*]} 
anzvib=1

j=0 
k=${ka}
dx=`echo "scale=20 ; (a(1)*8)*${anzvib}/(${anzsh})" | funcs.bc`
#echo "dx: ${dx}"
#exit 
aamp=0 
while [ ${j} -le ${anzsh} ] ; do
 amp=`echo "scale=8 ; ${ampp}*s(${dx}*${j})" | bc -l`
 j=${j}+1
 if ${unq} ; then
  amp=`echo "scale=8 ; ${j}-1-${anzsh}/2" | bc -l`
  amp=`echo "scale=8 ; 4*${ampp}/${anzsh}/${anzsh}*${amp}*${amp}*sgn(${amp})" | funcs.bc`
#  kamp=`echo "scale=8 ; int((${amp})*10000*sgn(${amp}))" | funcs.bc`
#  kaamp=`echo "scale=8 ; int((${aamp})*10000*sgn(${aamp}))" | funcs.bc`
#  aamp=${amp}
#  echo "kamp: ${kamp} ; kaamp: ${kaamp}"
#  [ ${kamp} -le ${kaamp} -a ${j} -ne 1 ] && continue 
 fi
 amp=`echo "scale=8 ; (${amp}+(${off}))" | bc -l`
 hk=`expi -i ${k} -l 7`
 echo "j: ${j} hk: ${hk} amp: ${amp}"
 lvvec=`mult ${amp} "${vvec}"`
# echo "lvvec: "${lvvec}
 lvvec=`add "${pvec}" "${lvvec}"`
# echo "lvvec: "${lvvec}
 ucss="${ucs}"
 if ${lela} ; then
  ucss="" ;   ja=1
  while [ ${ja} -le 3 ]  ; do # 
   c=(`echo -e  ${ela}  | head -${ja} | tail -1`)
   kk=${ja}-1
   echo "c: ${c[*]}"
   c[${kk}]="(${c[${kk}]}-1)"
# echo "c: ${c[*]}"
   c=(`mult $amp "${c[*]}"`)
# echo "c: ${c[*]}"
   c[${kk}]="(${c[${kk}]}+1)"
# echo "c: ${c[*]}"
  for p in 1 2 3 ; do 
   zi=`echo -e "${ucs}" | awk -v a=$p '{ print $a }'`
#  echo "c: ${c[*]}" "zi: "${zi} 
   ucss="${ucss} "`vmult "${c[*]}" "${zi}"`
  done 
  ucss="${ucss}\n"
  ja=${ja}+1  
 done
 ucss=`echo -e "${ucss}"`
 echo "ucss: ${ucss}"  
 fi
 ucsso=`echo "${ucss}" | sed 's/\n/\\n/g'`
 otp "${ivec}" "${lvvec}" ${ofi}_${hk}.xyz ${k} ${amp} "${ucsso}" "${mass}"& 
 k=${k}+1
done
wait

cat ${ofi}_[0-9][0-9]*.xyz  > ${ofi}.xyz 
rm ${ofi}_[0-9][0-9]*.xyz
