#!/bin/bash
# routine to
# Paolo Giannozzi giannozz at democritos.it On Thu, 2012-12-13 at 11:25 +0100
# header of matdyn.f90:
#  flvec output file for normalized phonon displacements (default: 'matdyn.modes'). The normalized phonon displacements are the eigenvectors divided by the mass and then normalized. As such they are not orthogonal.

typeset -i i j m k

ifi=matdyn.modes 
pfi=snap.xyz
phon=true
j0=0

while getopts i:p:j:o opt
do
 case $opt in
  i) ifi="${OPTARG}" # file containing vibrations
  ;;
  p) pfi="${OPTARG}" # file containing positions
  ;;
  j) j0="${OPTARG}" # do  decomposition for a specified index 
  ;;
  o) phon=false
  ;;
 esac
done ; OPTID=0

. integer_lib

hstr=`sed '1,2d ; s/ /  /g ; s/H /1.0079/;s/He /4.0026/;s/Li /6.941/;s/Be /9.0122/;s/B /10.811/;s/C /12.0107/;s/N /14.0067/;s/O /15.9994/;s/F /18.9984/;s/Ne /20.1797/;s/Na /22.9897/;s/Mg /24.305/;s/Al /26.9815/;s/Si /28.0855/;s/P /30.9738/;s/S /32.065/;s/Cl /35.453/;s/Ar /39.948/;s/K /39.0983/;s/Ca /40.078/;s/Sc /44.9559/;s/Ti /47.867/;s/V /50.9415/;s/Cr /51.9961/;s/Mn /54.938/;s/Fe /55.845/;s/Co /58.9332/;s/Ni /58.6934/;s/Cu /63.546/;s/Zn /65.39/;s/Ga /69.723/;s/Ge /72.64/;s/As /74.9216/;s/Se /78.96/;s/Br /79.904/;s/Kr /83.8/;s/Rb /85.4678/;s/Sr /87.62/;s/Y /88.9059/;s/Zr /91.224/;s/Nb /92.9064/;s/Mo /95.94/;s/Ru /101.07/;s/Rh /102.9055/;s/Pd /106.42/;s/Ag /107.8682/;s/Cd /112.411/;s/In /114.818/;s/Sn /118.71/;s/Sb /121.76/;s/Te /127.6/;s/I /126.9045/;s/Xe /131.293/;s/Cs /132.9055/;s/Ba /137.327/;s/La /138.9055/;s/Ce /140.116/;s/Pr /140.9077/;s/Nd /144.24/;s/Sm /150.36/;s/Eu /151.964/;s/Gd /157.25/;s/Tb /158.9253/;s/Dy /162.5/;s/Ho /164.9303/;s/Er /167.259/;s/Tm /168.9342/;s/Yb /173.04/;s/Lu /174.967/;s/Hf /178.49/;s/Ta /180.9479/;s/W /183.84/;s/Re /186.207/;s/Os /190.23/;s/Ir /192.217/;s/Pt /195.078/;s/Au /196.9665/;s/Hg /200.59/;s/Tl /204.3833/;s/Pb /207.2/;s/Bi /208.9804/;s/Th /232.0381/;s/Pa /231.0359/;s/U /238.028/ ' $pfi | awk '{ print $1 " "$1 " " $1 }'`
c=(${hstr})
echo "c: ${c[*]}"

if [[ ${ifi} == "matdyn.modes" ]] ; then 
 posa=(`sed -n '/\*\*\*\*/= ' ${ifi}`)
 echo "posa: ${posa[*]}"
 hstr=`cat ${ifi}`
 echo "${hstr}" | sed -n 1,${posa[1]}p  > .hef.$$ 
 ifi=.hef.$$
 head -5 $ifi
 tail -5 $ifi
 posl=(`sed -n '/omega/= ; /freq/= ; $=' ${ifi}`)
else  # MOLVIB
 posl=(`sed -n '/vibration/= ; $=' ${ifi}`)
fi
 
echo "#posl: ${#posl[*]}, ${posl[*]}"


j=${j0} ;  
m=${#posl[*]}-1
while [ ${j} -lt ${m} ] ; do
 i=${j}+1 ; echo $i ${posl[${j}]},${posl[${i}]}
 hstr=`sed -n ${posl[${j}]},${posl[${i}]}p ${ifi}`
# i=`echo "${hstr}" | head -1 | sed 's/(/ / ; s/)/ /' | awk '{ print $2 }'`
 hj=`expi -i $j`
 lifi=vib_${hj}.dat
 if [[ ${ifi} == "".hef.$$ ]] ; then 
  echo "${hstr}" | sed '/omega/d ; /freq/d ; /\*\*\*\*/d ; s/(/ /g ; s/)/ /g; ' | awk '{ print $1" "$3" "$5 }' > ${lifi}
 else
  echo "${hstr}" | sed '/vib/d  ; s/(/ /g ; s/)/ /g; ' | awk '{ print $1" "$2" "$3 }' > ${lifi}
 fi
# cat ${lifi}
 stm="0" ;  
 d=`cat ${lifi}` ; k=0 ; tdp=0
  for fon in ${d} ; do #scale coordinates by mass and calculate eq_mass
   if   ${phon}   ; then
    d[${k}]="(${fon})" 
   else
    d[${k}]="(${fon}*sqrt(${c[k]}))"
   fi
   tdp="${tdp}+(${d[${k}]})*(${d[${k}]})"
   k=${k}+1  
  done                                                                 

  k=0 ; i=1 ; mum=0 ;
 # echo "d: ${d[*]}" 
  for fon in ${d[*]} ; do 
  d[${k}]=`echo "scale=8 ; ${fon}/sqrt(${tdp})" | funcs.bc`
  mum="${mum}+(${d[${k}]})*(${d[${k}]})*${c[k]}"
#    d[${k}]="${fon}/${mum}"
   if [ ${i} -gt 2 ] ; then 
    d[${k}]="${d[${k}]} \n"
    i=0
   fi
   k=${k}+1 ; i=${i}+1
  done                                                # atomic mass unit to electron mass: 1822.88839
 mum=`echo "scale=8 ; ((${mum})*1822.88839) "  | bc ` # angst to bohr: 1.88973
# echo "mum: ${mum}"
 hhstr=`echo "${hstr}" | sed -n '/freq/p ; /omega/p' | sed 's/freq/omega/g' | head -1 `
 if   ${phon}   ; then
  echo -e "#mass 0 phonon ${hhstr}\n ${d[*]}" > ${lifi} # mass can be determined only by animate.bash
 else
  echo -e "#mass 0 orthogonal ${hhstr}\n ${d[*]}" > ${lifi}
 fi
 cat $lifi 
 j=${j}+1 
# [ ${j} -gt 3 ] && exit
# exit
[ ! ${j0} -eq 0 ] && exit
done
rm .hef.$$
exit
     diagonalizing the dynamical matrix ...

 q =       0.0000      0.0000      0.0000
 **************************************************************************
     omega( 1) =      -3.486914 [THz] =    -116.311696 [cm-1]
 (  0.000000   0.000000     0.000000   0.000000     0.000000   0.000000   )
 (  0.000000   0.000000     0.000000   0.000000     0.000000   0.000000   )
 (  0.000000   0.000000     0.348496   0.000000     0.000000   0.000000   )
 (  0.000000   0.000000    -0.351370   0.000000     0.000000   0.000000   )
 (  0.000000   0.000000     0.351370   0.000000     0.000000   0.000000   )
 (  0.000000   0.000000    -0.348496   0.000000     0.000000   0.000000   )
 (  0.000000   0.000000     0.000000   0.000000    -0.251673   0.000000   )
 (  0.000000   0.000000     0.000000   0.000000    -0.251673   0.000000   )
 (  0.000000   0.000000     0.000000   0.000000     0.251673   0.000000   )
 (  0.000000   0.000000     0.000000   0.000000     0.251673   0.000000   )
 (  0.000000   0.000000     0.000000   0.000000     0.253233   0.000000   )
 (  0.000000   0.000000     0.000000   0.000000    -0.253233   0.000000   )
 (  0.000000   0.000000     0.000000   0.000000     0.253233   0.000000   )
 (  0.000000   0.000000     0.000000   0.000000    -0.253233   0.000000   )
 (  0.000000   0.000000     0.000000   0.000000     0.000000   0.000000   )
 (  0.000000   0.000000     0.009854   0.000000     0.000000   0.000000   )
 (  0.000000   0.000000    -0.009854   0.000000     0.000000   0.000000   )
 (  0.000000   0.000000    -0.007715   0.000000     0.000000   0.000000   )
 (  0.000000   0.000000     0.007715   0.000000     0.000000   0.000000   )
 (  0.000000   0.000000     0.000000   0.000000     0.000000   0.000000   )
     omega( 2) =      -3.486804 [THz] =    -116.308051 [cm-1]
 (  0.000000   0.000000     0.000000   0.000000     0.000000   0.000000   )
 (  0.000000   0.000000     0.000000   0.000000     0.000000   0.000000   )
 ( -0.351643   0.000000     0.000000   0.000000     0.000000   0.000000   )
 (  0.353204   0.000000     0.000000   0.000000     0.000000   0.000000   )
