#!/bin/bash
# routine to

typeset -i i j

ifil=dipole-MOLECULE_0001_00001HOMO-1_0.cube 
export tachyon="/share/lib/vmd/tachyon_LINUXAMD64" 
export tachyon="/home/install/vmd-1.8.6/lib/tachyon/tachyon_LINUXAMD64"
jj=false
ival=0.002500
molstate=true
lrg="640 512"
queu=serial

while getopts i:jv:m:s: opt
do
 case $opt in
  i) ifil="${OPTARG}"
  ;;
  j) jj=true              # submit job for final rendering
  ;;
  v) ival="${OPTARG}"     # isovalue for isosurface
  ;;
  m) molstate="${OPTARG}" # plot an electron state with phase information
  ;;
  s) lrg="${OPTARG}" # large : "1280 1024" # size of the image
  ;;
  q) queu="${OPTARG}" # queue to submit rendering image
  ;;
 esac
done ; OPTIND=0
SCRIPT=`readlink -f $0` ; SCRIPTPATH=`dirname $SCRIPT` 
if [ ${molstate} ] ; then
 col1id=0
 addt="mol representation Isosurface -${ival} 0.000000 0.000000 0.000000 1 1 \n
mol color ColorID 1 \nmol selection {all} \nmol material Opaque \nmol addrep top\n
mol selupdate 2 top 0 \nmol colupdate 2 top 0 \n mol scaleminmax top 2 0.000000 0.000000 \nmol smoothrep top 2 0"
else
 addt=""
 col1id=7
fi
addt=`echo -e ${addt}`
#echo "${addt}"

while read lin ; do
echo "${lin}" >> .hef.$$.vmd
done << EOF
#!/usr/local/bin/vmd
color scale method RWB
after idle {
  color Name H white
  color Type H white
  color Name C silver
  color Type C silver
  color Name N green
  color Type N green
}
# Display settings
display eyesep       0.065000
display focallength  2.000000
display height       6.000000
display distance     -2.000000
display projection   Orthographic
display nearclip set 0.010000
display farclip  set 10.000000
display depthcue   off
display cuestart   0.500000
display cueend     10.000000
display cuedensity 0.400000
display cuemode    Exp2
mol new MMOFF.cube type cube first 0 last -1 step 1 filebonds 1 autobonds 1 waitfor all
mol delrep 0 top
mol representation CPK 0.200000 0.100000 23.000000 21.000000
mol color Name
mol selection {all}
mol material Diffuse
mol addrep top
mol selupdate 0 top 0
mol colupdate 0 top 0
mol scaleminmax top 0 0.000000 0.000000
mol smoothrep top 0 0
mol drawframes top 0 {now}
mol representation Isosurface ${ival} 0.000000 0.000000 0.000000 1 1
mol color ColorID ${col1id} 
mol selection {all}
mol material Opaque
mol addrep top
mol selupdate 1 top 0
mol colupdate 1 top 0
mol scaleminmax top 1 0.000000 0.000000
mol smoothrep top 1 0
${addt} 
mol rename top MMOFF.cube
set topmol [molinfo top]
mol top \$topmol
unset topmol
scale by 1.5
color Display Background white 
display projection Orthographic
display resize ${lrg} 
axes location off
set viewpoints([molinfo top]) {{{1.000000 0.000000 0.000000 -6.415436} {0.000000 1.000000 0.000000 -6.323853} {0.000000 0.000000 1.000000 -8.980606} {0.000000 0.000000 0.000000 1.000000}} {{-0.103794 0.081819 -0.991228 0.000000} {-0.406830 -0.912918 -0.032755 0.000000} {-0.907587 0.399860 0.128042 0.000000} {0.000000 0.000000 0.000000 1.000000}} {{0.292509 0.000000 0.000000 0.000000} {0.000000 0.292509 0.000000 0.000000} {0.000000 0.000000 0.292509 0.000000} {0.000000 0.000000 0.000000 1.000000}} {{1.000000 0.000000 0.000000 0.000000} {0.000000 1.000000 0.000000 0.000000} {0.000000 0.000000 1.000000 0.000000} {0.000000 0.000000 0.000000 1.000000}}}
lappend viewplist [molinfo top]
set topmol [molinfo top]
# done with molecule 0
foreach v \$viewplist {
  molinfo \$v set {center_matrix rotate_matrix scale_matrix global_matrix} \$viewpoints(\$v)
}
foreach v \$fixedlist {
  molinfo \$v set fixed 1
}
unset viewplist
unset fixedlist
render Tachyon MMOFF.dat
EOF
#> mol drawframes top 2 {now}


j=0
for fon in ${ifil} ; do
 name=(`echo ${fon} | awk -F . '{ print $1 " "$NF }'`)
 echo $fon $name $j
 sed "s/cube/${name[1]}/g ; s/MMOFF/${name}/g ; " .hef.$$.vmd > .hef_${j}.$$.vmd
#cat .hef.vmd 
#cat .hef_${j}.vmd
 vmd -dispdev text -eofexit < .hef_${j}.$$.vmd > .hef_${j}.$$.log
# sleep 3
 if ${jj} ; then
  ~/viz/ssmit -i ${name}.dat -q ${queu} 
 else
  ${tachyon} -auto_skylight 0.8 ${name}.dat -o ${name}.tga
  rm ${name}.dat
 fi
 j=${j}+1
done

#rm .hef*.$$.*
exit
